@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2014 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -36,12 +36,13 @@
 #include <string.h>
 
 #include "oval_definitions_impl.h"
-#include "oval_collection_impl.h"
+#include "adt/oval_collection_impl.h"
 #include "oval_parser_impl.h"
-#include "oval_string_map_impl.h"
+#include "adt/oval_string_map_impl.h"
 
 #include "common/util.h"
 #include "common/debug_priv.h"
+#include "common/elements.h"
 #include "common/_error.h"
 
 /***************************************************************************/
@@ -201,14 +202,14 @@ static int _oval_affected_parse_tag(xmlTextReaderPtr reader, struct oval_parser_
 
 	if (strcmp((char *)tagname, "platform") == 0) {
 		char *platform = NULL;
-		return_code = oval_parser_text_value(reader, context, &oval_text_consumer, &platform);
+		return_code = oscap_parser_text_value(reader, &oscap_text_consumer, &platform);
 		if (platform != NULL) {
 			oval_affected_add_platform(affected, platform);
 			oscap_free(platform);
 		}
 	} else if (strcmp((char *)tagname, "product") == 0) {
 		char *product = NULL;
-		return_code = oval_parser_text_value(reader, context, &oval_text_consumer, &product);
+		return_code = oscap_parser_text_value(reader, &oscap_text_consumer, &product);
 		if (product != NULL) {
 			oval_affected_add_product(affected, product);
 			oscap_free(product);
