@@ -38,6 +38,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
+#include <stdbool.h>
 
 #include "oval_definitions_impl.h"
 #include "adt/oval_collection_impl.h"
@@ -48,6 +49,8 @@
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
+#include "common/oscap_string.h"
+#include "oval_glob_to_regex.h"
 #if defined USE_REGEX_PCRE
 #include <pcre.h>
 #elif defined USE_REGEX_POSIX
@@ -162,6 +165,13 @@ typedef struct oval_component_SPLIT {
 	char *delimiter;	/*type==OVAL_COMPONENT_SPLIT */
 } oval_component_SPLIT_t;
 
+typedef struct oval_component_GLOB {
+	struct oval_definition_model *model;
+	oval_component_type_t type;
+	struct oval_collection *function_components;	/*type==OVAL_COMPONENT_FUNCTION */
+	bool glob_noescape;	/*type==OVAL_COMPONENT_GLOB */
+} oval_component_GLOB_t;
+
 typedef struct oval_component_SUBSTRING {
 	struct oval_definition_model *model;
 	oval_component_type_t type;
@@ -200,6 +210,8 @@ void oval_component_to_print(struct oval_component *component, char *indent, int
 	OVAL_FUNCTION_REGEX_CAPTURE = OVAL_FUNCTION + 8,
 	OVAL_FUNCTION_ARITHMETIC = OVAL_FUNCTION + 9
 	OVAL_FUNCTION_COUNT = OVAL_FUNCTION + 10,
+	OVAL_FUNCTION_GLOB_TO_REGEX = OVAL_FUNCTION + 12,
+
  */
 static const struct oscap_string_map _OVAL_COMPONENT_MAP[] = {
 	{OVAL_COMPONENT_LITERAL, "literal_component"},
@@ -220,6 +232,7 @@ static const struct oscap_string_map _OVAL_FUNCTION_MAP[] = {
 	{OVAL_FUNCTION_ARITHMETIC, "arithmetic"},
 	{OVAL_FUNCTION_COUNT, "count"},
 	{OVAL_FUNCTION_UNIQUE, "unique"},
+	{OVAL_FUNCTION_GLOB_TO_REGEX, "glob_to_regex"},
 	{0, NULL}
 };
 
@@ -451,6 +464,30 @@ void oval_component_set_split_delimiter(struct oval_component *component, char *
 	}
 }
 
+bool oval_component_get_glob_to_regex_glob_noescape(struct oval_component *component)
+{
+	__attribute__nonnull__(component);
+
+	/* type == OVAL_COMPONENT_GLOB */
+	bool glob_noescape;
+	if (component->type == OVAL_FUNCTION_GLOB_TO_REGEX) {
+		oval_component_GLOB_t *glob_to_regex = (oval_component_GLOB_t *) component;
+		glob_noescape = glob_to_regex->glob_noescape;
+	} else
+		glob_noescape = false;
+	return glob_noescape;
+}
+
+void oval_component_set_glob_to_regex_glob_noescape(struct oval_component *component, bool glob_noescape) {
+	__attribute__nonnull__(component);
+
+	/* type == OVAL_COMPONENT_GLOB */
+	if (component->type == OVAL_FUNCTION_GLOB_TO_REGEX) {
+		oval_component_GLOB_t *glob_to_regex = (oval_component_GLOB_t *) component;
+		glob_to_regex->glob_noescape = glob_noescape;
+	}
+}
+
 int oval_component_get_substring_start(struct oval_component *component)
 {
 	__attribute__nonnull__(component);
@@ -646,6 +683,15 @@ struct oval_component *oval_component_new(struct oval_definition_model *model, o
 					split->delimiter = NULL;
 				};
 				break;
+			case OVAL_FUNCTION_GLOB_TO_REGEX:{
+					oval_component_GLOB_t *glob_to_regex = (oval_component_GLOB_t *)
+					    (function = (oval_component_FUNCTION_t *)
+					     oscap_alloc(sizeof(oval_component_GLOB_t)));
+					if (glob_to_regex == NULL)
+						return NULL;
+					glob_to_regex->glob_noescape = false;
+				};
+				break;
 			case OVAL_FUNCTION_SUBSTRING:{
 					oval_component_SUBSTRING_t *substring = (oval_component_SUBSTRING_t *)
 					    (function = (oval_component_FUNCTION_t *)
@@ -769,6 +815,11 @@ struct oval_component *oval_component_clone(struct oval_definition_model *new_mo
 				oval_component_set_split_delimiter(new_component, delimiter);
 		}
 		break;
+	case OVAL_FUNCTION_GLOB_TO_REGEX:{
+			bool glob_noescape = oval_component_get_glob_to_regex_glob_noescape(old_component);
+			oval_component_set_glob_to_regex_glob_noescape(new_component, glob_noescape);
+		}
+		break;
 	case OVAL_FUNCTION_SUBSTRING:{
 			int length = oval_component_get_substring_length(old_component);
 			oval_component_set_substring_length(new_component, length);
@@ -842,6 +893,7 @@ void oval_component_free(struct oval_component *component)
 			regex->pattern = NULL;
 		};
 		break;
+	case OVAL_FUNCTION_GLOB_TO_REGEX:
 	case OVAL_FUNCTION_CONCAT:
 	case OVAL_FUNCTION_COUNT:
 	case OVAL_FUNCTION_UNIQUE:
@@ -996,6 +1048,21 @@ static int _oval_component_parse_SPLIT_tag(xmlTextReaderPtr reader,
 	return _oval_component_parse_FUNCTION_tag(reader, context, component);
 }
 
+static int _oval_component_parse_GLOB_TO_REGEX_tag(xmlTextReaderPtr reader,
+					   struct oval_parser_context *context, struct oval_component *component)
+{
+
+	__attribute__nonnull__(component);
+
+	oval_component_GLOB_t *glob_to_regex = (oval_component_GLOB_t *) component;
+	char *glob_noescape = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "glob_noescape");
+	glob_to_regex->glob_noescape = (glob_noescape != NULL && strcmp(glob_noescape, "true") == 0);
+	oscap_free(glob_noescape);
+
+	return _oval_component_parse_FUNCTION_tag(reader, context, component);
+}
+
+
 static int _oval_component_parse_SUBSTRING_tag(xmlTextReaderPtr reader,
 					       struct oval_parser_context *context, struct oval_component *component)
 {
@@ -1089,6 +1156,9 @@ int oval_component_parse_tag(xmlTextReaderPtr reader,
 	} else if (strcmp(tagname, "escape_regex") == 0) {
 		component = oval_component_new(model, OVAL_FUNCTION_ESCAPE_REGEX);
 		return_code = _oval_component_parse_FUNCTION_tag(reader, context, component);
+	} else if (strcmp(tagname, "glob_to_regex") == 0) {
+		component = oval_component_new(model, OVAL_FUNCTION_GLOB_TO_REGEX);
+		return_code = _oval_component_parse_GLOB_TO_REGEX_tag(reader, context, component);
 	} else if (strcmp(tagname, "split") == 0) {
 		component = oval_component_new(model, OVAL_FUNCTION_SPLIT);
 		return_code = _oval_component_parse_SPLIT_tag(reader, context, component);
@@ -1212,6 +1282,14 @@ xmlNode *oval_component_to_dom(struct oval_component *component, xmlDoc * doc, x
 			char * delimiter = oval_component_get_split_delimiter(component);
 			xmlNewProp(component_node, BAD_CAST "delimiter", BAD_CAST delimiter);
 		} break;
+	case OVAL_FUNCTION_GLOB_TO_REGEX:{
+			bool glob_noescape = oval_component_get_glob_to_regex_glob_noescape(component);
+			if (glob_noescape) {
+				xmlNewProp(component_node, BAD_CAST "glob_noescape", BAD_CAST "true");
+			} else {
+				xmlNewProp(component_node, BAD_CAST "glob_noescape", BAD_CAST "false");
+			}
+		} break;
 	case OVAL_FUNCTION_CONCAT:
 	case OVAL_FUNCTION_ESCAPE_REGEX:
 		break;
@@ -1973,6 +2051,41 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ESCAPE_REGEX(oval
 	return flag;
 }
 
+
+
+
+
+static oval_syschar_collection_flag_t _oval_component_evaluate_GLOB_TO_REGEX(oval_argu_t *argu,
+										struct oval_component *component,
+										struct oval_collection *value_collection)
+{
+	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
+	bool glob_noescape = oval_component_get_glob_to_regex_glob_noescape(component);
+	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
+	if (oval_component_iterator_has_more(subcomps)) {	//Only first component is considered
+		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
+		struct oval_collection *subcoll = oval_collection_new();
+		flag = oval_component_eval_common(argu, subcomp, subcoll);
+		struct oval_value_iterator *values = (struct oval_value_iterator *)oval_collection_iterator(subcoll);
+		while (oval_value_iterator_has_more(values)) {
+			struct oval_value *value = oval_value_iterator_next(values);
+			char *text = oval_value_get_text(value);
+			char *string = oval_glob_to_regex(text, glob_noescape);
+			if (string == NULL) {
+				flag = SYSCHAR_FLAG_ERROR;
+				break;
+			}
+			value = oval_value_new(OVAL_DATATYPE_STRING, string);
+			oscap_free(string);
+			oval_collection_add(value_collection, value);
+		}
+		oval_value_iterator_free(values);
+		oval_collection_free_items(subcoll, (oscap_destruct_func) oval_value_free);
+	}
+	oval_component_iterator_free(subcomps);
+	return flag;
+}
+
 static oval_syschar_collection_flag_t _oval_component_evaluate_REGEX_CAPTURE(oval_argu_t *argu,
 									     struct oval_component *component,
 									     struct oval_collection *value_collection)
@@ -2107,7 +2220,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC_rec(st
 		} else if (dt == OVAL_DATATYPE_FLOAT) {
 			new_val = (double) oval_value_get_float(ov);
 		} else {
-			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected value type: %s.\n", oval_datatype_get_text(dt));
+			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected value type: %s.", oval_datatype_get_text(dt));
 			oval_value_iterator_free(val_itr);
 			return SYSCHAR_FLAG_ERROR;
 		}
@@ -2117,7 +2230,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC_rec(st
 		} else if (op == OVAL_ARITHMETIC_MULTIPLY) {
 			new_val *= val;
 		} else {
-			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected arithmetic operation: %s.\n", oval_arithmetic_operation_get_text(op));
+			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected arithmetic operation: %s.", oval_arithmetic_operation_get_text(op));
 			oval_value_iterator_free(val_itr);
 			return SYSCHAR_FLAG_ERROR;
 		}
@@ -2143,7 +2256,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC(oval_a
 
 	op = oval_component_get_arithmetic_operation(component);
 	if (op != OVAL_ARITHMETIC_ADD && op != OVAL_ARITHMETIC_MULTIPLY) {
-		oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected arithmetic operation: %s.\n", oval_arithmetic_operation_get_text(op));
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected arithmetic operation: %s.", oval_arithmetic_operation_get_text(op));
 		return SYSCHAR_FLAG_ERROR;
 	}
 
@@ -2177,7 +2290,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC(oval_a
 		} else if (datatype == OVAL_DATATYPE_FLOAT) {
 			val = (double) oval_value_get_float(ov);
 		} else {
-			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected value type: %s.\n", oval_datatype_get_text(datatype));
+			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected value type: %s.", oval_datatype_get_text(datatype));
 			flag = SYSCHAR_FLAG_ERROR;
 			goto cleanup;
 		}
@@ -2217,7 +2330,8 @@ static _oval_component_evaluator *_component_evaluators[] = {
 	_oval_component_evaluate_ARITHMETIC,
 	_oval_component_evaluate_COUNT,
 	_oval_component_evaluate_UNIQUE,
-	NULL
+	_oval_component_evaluate_GLOB_TO_REGEX,
+	NULL,
 };
 
 static oval_syschar_collection_flag_t oval_component_eval_common(oval_argu_t *argu,
@@ -2234,7 +2348,7 @@ static oval_syschar_collection_flag_t oval_component_eval_common(oval_argu_t *ar
 	if (evaluator) {
 		flag = (*evaluator) (argu, component, value_collection);
 	} else {
-		oscap_seterr(OSCAP_EFAMILY_OVAL, "Component type %d not supported.\n", type);
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "Component type %d not supported.", type);
 	}
 	return flag;
 }
