@@ -338,6 +338,7 @@ static const struct oscap_string_map OVAL_COMPONENT_TYPE_MAP[] = {
 	{OVAL_FUNCTION_REGEX_CAPTURE, "regex_capture"},
 	{OVAL_FUNCTION_ARITHMETIC, "arithmetic"},
 	{OVAL_FUNCTION_COUNT, "count"},
+	{OVAL_FUNCTION_GLOB_TO_REGEX, "glob_to_regex"},
 	{OVAL_ENUMERATION_INVALID, NULL}
 };
 
@@ -485,6 +486,8 @@ static const struct oscap_string_map OVAL_SUBTYPE_LINUX_MAP[] = {
         {OVAL_LINUX_RPMVERIFYPACKAGE, "rpmverifypackage"},
         {OVAL_LINUX_SELINUXBOOLEAN, "selinuxboolean"},
         {OVAL_LINUX_SELINUXSECURITYCONTEXT, "selinuxsecuritycontext"},
+        {OVAL_LINUX_SYSTEMDUNITPROPERTY, "systemdunitproperty"},
+        {OVAL_LINUX_SYSTEMDUNITDEPENDENCY, "systemdunitdependency"},
 	{OVAL_SUBTYPE_UNKNOWN, NULL}
 };
 
