@@ -48,6 +48,7 @@
 #include "_oval_probe_handler.h"
 #include "oval_probe_meta.h"
 #include "oval_probe_ext.h"
+#include "collectVarRefs_impl.h"
 
 oval_probe_meta_t OSCAP_GSYM(__probe_meta)[] = {
         { OVAL_SUBTYPE_SYSINFO, "system_info", &oval_probe_sys_handler, OVAL_PROBEMETA_EXTERNAL, "probe_system_info" },
@@ -74,6 +75,8 @@ oval_probe_meta_t OSCAP_GSYM(__probe_meta)[] = {
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_RPMVERIFYPACKAGE, "rpmverifypackage"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_SELINUXBOOLEAN, "selinuxboolean"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_SELINUXSECURITYCONTEXT, "selinuxsecuritycontext"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_SYSTEMDUNITPROPERTY, "systemdunitproperty"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_SYSTEMDUNITDEPENDENCY, "systemdunitdependency"),
         OVAL_PROBE_EXTERNAL(OVAL_SOLARIS_ISAINFO, "isainfo"),
         OVAL_PROBE_EXTERNAL(OVAL_UNIX_FILE, "file"),
         OVAL_PROBE_EXTERNAL(OVAL_UNIX_INTERFACE, "interface"),
@@ -193,158 +196,6 @@ oval_subtype_t oval_str_to_subtype(const char *str)
         return (d == NULL ? OVAL_SUBTYPE_UNKNOWN : d->type);
 }
 
-static void _obj_collect_var_refs(struct oval_object *obj, struct oval_string_map *vm);
-static void _var_collect_var_refs(struct oval_variable *var, struct oval_string_map *vm);
-
-static void _comp_collect_var_refs(struct oval_component *comp, struct oval_string_map *vm)
-{
-	struct oval_object *obj;
-	struct oval_variable *var;
-	struct oval_component_iterator *cmp_itr;
-
-	switch (oval_component_get_type(comp)) {
-	case OVAL_COMPONENT_OBJECTREF:
-		obj = oval_component_get_object(comp);
-		_obj_collect_var_refs(obj, vm);
-		break;
-	case OVAL_COMPONENT_VARREF:
-		var = oval_component_get_variable(comp);
-		_var_collect_var_refs(var, vm);
-		break;
-	case OVAL_FUNCTION_ARITHMETIC:
-	case OVAL_FUNCTION_BEGIN:
-	case OVAL_FUNCTION_CONCAT:
-	case OVAL_FUNCTION_END:
-	case OVAL_FUNCTION_ESCAPE_REGEX:
-	case OVAL_FUNCTION_REGEX_CAPTURE:
-	case OVAL_FUNCTION_SPLIT:
-	case OVAL_FUNCTION_SUBSTRING:
-	case OVAL_FUNCTION_TIMEDIF:
-		cmp_itr = oval_component_get_function_components(comp);
-		while (oval_component_iterator_has_more(cmp_itr)) {
-			struct oval_component *cmp;
-
-			cmp = oval_component_iterator_next(cmp_itr);
-			_comp_collect_var_refs(cmp, vm);
-		}
-		oval_component_iterator_free(cmp_itr);
-		break;
-	default:
-		break;
-	}
-}
-
-static void _var_collect_var_refs(struct oval_variable *var, struct oval_string_map *vm)
-{
-	char *var_id;
-
-	var_id = oval_variable_get_id(var);
-	oval_string_map_put(vm, var_id, var);
-
-	if (oval_variable_get_type(var) == OVAL_VARIABLE_LOCAL) {
-		struct oval_component *comp;
-
-		comp = oval_variable_get_component(var);
-		_comp_collect_var_refs(comp, vm);
-	}
-}
-
-static void _ent_collect_var_refs(struct oval_entity *ent, struct oval_string_map *vm)
-{
-	if (oval_entity_get_varref_type(ent) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
-		struct oval_variable *var;
-
-		var = oval_entity_get_variable(ent);
-		_var_collect_var_refs(var, vm);
-	}
-}
-
-static void _ste_collect_var_refs(struct oval_state *ste, struct oval_string_map *vm)
-{
-	struct oval_state_content_iterator *cont_itr;
-
-	cont_itr = oval_state_get_contents(ste);
-	while (oval_state_content_iterator_has_more(cont_itr)) {
-		struct oval_state_content *cont;
-		struct oval_entity *ent;
-
-		cont = oval_state_content_iterator_next(cont_itr);
-		ent = oval_state_content_get_entity(cont);
-		_ent_collect_var_refs(ent, vm);
-	}
-	oval_state_content_iterator_free(cont_itr);
-}
-
-static void _set_collect_var_refs(struct oval_setobject *set, struct oval_string_map *vm)
-{
-	struct oval_setobject_iterator *subset_itr;
-	struct oval_object_iterator *obj_itr;
-	struct oval_filter_iterator *fil_itr;
-
-	switch (oval_setobject_get_type(set)) {
-	case OVAL_SET_AGGREGATE:
-		subset_itr = oval_setobject_get_subsets(set);
-		while (oval_setobject_iterator_has_more(subset_itr)) {
-			struct oval_setobject *subset;
-
-			subset = oval_setobject_iterator_next(subset_itr);
-			_set_collect_var_refs(subset, vm);
-		}
-		oval_setobject_iterator_free(subset_itr);
-		break;
-	case OVAL_SET_COLLECTIVE:
-		obj_itr = oval_setobject_get_objects(set);
-		while (oval_object_iterator_has_more(obj_itr)) {
-			struct oval_object *obj;
-
-			obj = oval_object_iterator_next(obj_itr);
-			_obj_collect_var_refs(obj, vm);
-		}
-		oval_object_iterator_free(obj_itr);
-		fil_itr = oval_setobject_get_filters(set);
-		while (oval_filter_iterator_has_more(fil_itr)) {
-			struct oval_filter *fil;
-			struct oval_state *ste;
-
-			fil = oval_filter_iterator_next(fil_itr);
-			ste = oval_filter_get_state(fil);
-			_ste_collect_var_refs(ste, vm);
-		}
-		oval_filter_iterator_free(fil_itr);
-		break;
-	default:
-		break;
-	}
-}
-
-static void _obj_collect_var_refs(struct oval_object *obj, struct oval_string_map *vm)
-{
-	struct oval_object_content_iterator *cont_itr;
-
-	cont_itr = oval_object_get_object_contents(obj);
-	while (oval_object_content_iterator_has_more(cont_itr)) {
-		struct oval_object_content *cont;
-		struct oval_entity *ent;
-		struct oval_setobject *set;
-
-		cont = oval_object_content_iterator_next(cont_itr);
-
-		switch (oval_object_content_get_type(cont)) {
-		case OVAL_OBJECTCONTENT_ENTITY:
-			ent = oval_object_content_get_entity(cont);
-			_ent_collect_var_refs(ent, vm);
-			break;
-		case OVAL_OBJECTCONTENT_SET:
-			set = oval_object_content_get_setobject(cont);
-			_set_collect_var_refs(set, vm);
-			break;
-		default:
-			break;
-		}
-	}
-	oval_object_content_iterator_free(cont_itr);
-}
-
 static void _syschar_add_bindings(struct oval_syschar *sc, struct oval_string_map *vm)
 {
 	struct oval_iterator *var_itr;
@@ -392,16 +243,25 @@ int oval_probe_query_object(oval_probe_session_t *psess, struct oval_object *obj
 
 	sysc = oval_syschar_model_get_syschar(model, oid);
 	if (sysc != NULL) {
-		oval_syschar_collection_flag_t sc_flg;
+		int variable_instance_hint = oval_syschar_get_variable_instance_hint(sysc);
+		if (oval_syschar_get_variable_instance_hint(sysc) != oval_syschar_get_variable_instance(sysc)) {
+			dI("Creating another syschar for variable_instance=%d)\n", variable_instance_hint);
+			sysc = oval_syschar_new(model, object);
+			oval_syschar_set_variable_instance(sysc, variable_instance_hint);
+			oval_syschar_set_variable_instance_hint(sysc, variable_instance_hint);
+		}
+		else {
+			oval_syschar_collection_flag_t sc_flg;
 
-		sc_flg = oval_syschar_get_flag(sysc);
+			sc_flg = oval_syschar_get_flag(sysc);
 
-		dI("Syschar already exists, flag: %u, '%s'.\n", sc_flg, oval_syschar_collection_flag_get_text(sc_flg));
+			dI("Syschar already exists, flag: %u, '%s'.\n", sc_flg, oval_syschar_collection_flag_get_text(sc_flg));
 
-		if (sc_flg != SYSCHAR_FLAG_UNKNOWN || (flags & OVAL_PDFLAG_NOREPLY)) {
-			if (out_syschar)
-				*out_syschar = sysc;
-			return 0;
+			if (sc_flg != SYSCHAR_FLAG_UNKNOWN || (flags & OVAL_PDFLAG_NOREPLY)) {
+				if (out_syschar)
+					*out_syschar = sysc;
+				return 0;
+			}
 		}
 	} else
 		sysc = oval_syschar_new(model, object);
@@ -428,7 +288,7 @@ int oval_probe_query_object(oval_probe_session_t *psess, struct oval_object *obj
 
 	if (!(flags & OVAL_PDFLAG_NOREPLY)) {
 		vm = oval_string_map_new();
-		_obj_collect_var_refs(object, vm);
+		oval_obj_collect_var_refs(object, vm);
 		_syschar_add_bindings(sysc, vm);
 		oval_string_map_free(vm, NULL);
 	}
@@ -579,6 +439,11 @@ static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_cri
         case OVAL_NODETYPE_EXTENDDEF:{
                         struct oval_definition *oval_def = oval_criteria_node_get_definition(cnode);
 			struct oval_criteria_node *node =  oval_definition_get_criteria(oval_def);
+			if (node == NULL) {
+				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Could not find extended definition: %s.",
+					oval_definition_get_id(oval_def));
+				return -1;
+			}
                         return oval_probe_query_criteria(sess, node);
                 }
                 break;
@@ -599,53 +464,47 @@ const oval_probe_meta_t * const oval_probe_meta_get(void)
 
 void oval_probe_meta_list(FILE *output, int flags)
 {
-    register size_t i;
-    const char *probe_dir;
-    char probe_path[PATH_MAX];
-    size_t probe_dirlen;
-    oval_probe_meta_t *meta = OSCAP_GSYM(__probe_meta);
+	register size_t i;
+	const char *probe_dir;
+	oval_probe_meta_t *meta = OSCAP_GSYM(__probe_meta);
+	size_t probe_dirlen;
+	char probe_path[PATH_MAX+1];
 
-    if (output == NULL)
-	output = stdout;
-#if 0
-    fprintf(output, "#\n# %-24s %-30s %s", "object", "probe name", "flags");
+	if (output == NULL)
+		output = stdout;
 
-    if (flags & OVAL_PROBEMETA_LIST_VERBOSE) {
-	fprintf(output, " %-5s %s\n#\n", "type", "path");
-    } else
-	fprintf(output, "\n#\n");
-#endif
-    probe_dir = oval_probe_ext_getdir();
-    assume_d(probe_dir != NULL, /* void */);
-    probe_dirlen = strlen(probe_dir);
-    assume_r(probe_dirlen + 1 < sizeof probe_path - 1, /* void */);
-    strncpy(probe_path, probe_dir, sizeof probe_path - 1);
+	probe_dir = oval_probe_ext_getdir();
+	assume_d(probe_dir != NULL, /* void */);
+	probe_dirlen = strlen(probe_dir);
+	assume_r(probe_dirlen + 1 <= PATH_MAX, /* void */);
 
-    probe_path[probe_dirlen  ] = '/';
-    probe_path[probe_dirlen+1] = '\0';
+	for (i = 0; i < OSCAP_GSYM(__probe_meta_count); ++i) {
+		if (meta[i].flags & OVAL_PROBEMETA_EXTERNAL) {
+			strncpy(probe_path, probe_dir, PATH_MAX);
+			probe_path[probe_dirlen] = '/';
+			probe_path[probe_dirlen+1] = '\0';
+			strncat(probe_path, meta[i].pname, PATH_MAX - strlen(probe_dir) - 1);
 
-    for (i = 0; i < OSCAP_GSYM(__probe_meta_count); ++i) {
-	if (meta[i].flags & OVAL_PROBEMETA_EXTERNAL) {
-	    strncpy(probe_path + probe_dirlen + 1,
-		    meta[i].pname,
-		    sizeof probe_path - probe_dirlen + 1);
-
-	    if (flags & OVAL_PROBEMETA_LIST_DYNAMIC) {
-		dI("Checking access to \"%s\"\n", probe_path);
-		if (access(probe_path, X_OK) != 0) {
-		    dW("access: errno=%d, %s\n", errno, strerror(errno));
-		    continue;
+			if (flags & OVAL_PROBEMETA_LIST_DYNAMIC) {
+				dI("Checking access to \"%s\"\n", probe_path);
+				if (access(probe_path, X_OK) != 0) {
+					dW("access: errno=%d, %s\n", errno, strerror(errno));
+					continue;
+				}
+			}
 		}
-	    }
+
+		fprintf(output, "%-28s %-28s", meta[i].stype, meta[i].pname);
+
+		if (flags & OVAL_PROBEMETA_LIST_VERBOSE) {
+			if (meta[i].flags & OVAL_PROBEMETA_EXTERNAL) {
+				fprintf(output, " %-5u %s\n", meta[i].otype, probe_path);
+			} else {
+				fprintf(output, " %-5u\n", meta[i].otype);
+			}
+		} else
+			fprintf(output, "\n");
 	}
 
-	fprintf(output, "%-28s %-28s", meta[i].stype, meta[i].pname);
-
-	if (flags & OVAL_PROBEMETA_LIST_VERBOSE) {
-	    fprintf(output, " %-5u %s\n", meta[i].otype, probe_path);
-	} else
-	    fprintf(output, "\n");
-    }
-
-    return;
+	return;
 }
