@@ -40,6 +40,13 @@ OSCAP_HIDDEN_START;
  */
 const struct xccdf_version_info* xccdf_detect_version_parser(xmlTextReaderPtr reader);
 
+/**
+ * Detects version from the xmlTextReader.
+ *
+ * The reader needs to be at the start of document.
+ */
+char *xccdf_detect_version_priv(xmlTextReader *reader);
+
 /**
  * Return true if the given namespace is supported XCCDF namespace.
  */
@@ -122,7 +129,8 @@ typedef enum {
 	XCCDFE_VALUE_VAL,
 	XCCDFE_VERSION,
 	XCCDFE_WARNING,
-	XCCDFE_BENCHMARK_REF,
+	/// benchmark ref and result benchmark have the same element name and namespace
+	XCCDFE_BENCHMARK_REF = XCCDFE_RESULT_BENCHMARK,
 
 	XCCDFE_END_
 } xccdf_element_t;
@@ -205,6 +213,8 @@ void xccdf_print_max(const char *str, int max, const char *ellipsis);
 void xccdf_print_max_text(const struct oscap_text *txt, int max, const char *ellipsis);
 void xccdf_print_textlist(struct oscap_text_iterator *txt, int depth, int max, const char *ellipsis);
 
+xmlNs *lookup_xccdf_ns(xmlDoc *doc, xmlNode *parent, const struct xccdf_version_info *version_info);
+
 OSCAP_HIDDEN_END;
 
 #endif
