@@ -1,5 +1,5 @@
 /*
- * Copyright 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2014 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -166,24 +166,78 @@ bool xccdf_policy_model_set_tailoring(struct xccdf_policy_model *model, struct x
  */
 struct xccdf_tailoring *xccdf_policy_model_get_tailoring(struct xccdf_policy_model *model);
 
+/**
+ * Get human readable title of given XCCDF Item. This finds title with best matching language
+ * and resolves <xccdf:sub> substitution in accordance with the given XCCDF Policy.
+ * @memberof xccdf_policy
+ * @param policy XCCDF Policy
+ * @param item XCCDF Item to query title from
+ * @param preferred_lang Language of your choice, Null value for the default.
+ * @returns plaintext C string which must be freed by caller
+ */
+char *xccdf_policy_get_readable_item_title(struct xccdf_policy *policy, struct xccdf_item *item, const char *preferred_lang);
+
+/**
+ * Get human readable description of given XCCDF Item. This function searches for description
+ * with the best matching language and resolves any inner <xccdf:sub> substitution (in accordance
+ * with the given XCCDF Policy.
+ * @memberof xccdf_policy
+ * @param policy XCCDF Policy
+ * @param item XCCDF Item to query description from
+ * @param preferred_lang Language of your choice, Null value for the default.
+ * @returns plaintext C string which must be freed by caller
+ */
+char *xccdf_policy_get_readable_item_description(struct xccdf_policy *policy, struct xccdf_item *item, const char *preferred_lang);
+
 /**
  * Registers an additional CPE dictionary for applicability testing
  * The one embedded in the evaluated XCCDF take precedence!
  */
+bool xccdf_policy_model_add_cpe_dict_source(struct xccdf_policy_model * model, struct oscap_source *source);
+
+/**
+ * Registers an additional CPE dictionary for applicability testing
+ * The one embedded in the evaluated XCCDF take precedence!
+ *
+ * @deprecated Deprecated in favor of @ref xccdf_policy_model_add_cpe_dict_source
+ */
 bool xccdf_policy_model_add_cpe_dict(struct xccdf_policy_model * model, const char * cpe_dict);
 
 /**
  * Registers an additional CPE lang model for applicability testing
  * The one embedded in the evaluated XCCDF take precedence!
  */
-bool xccdf_policy_model_add_cpe_lang_model(struct xccdf_policy_model * model, const char * cpe_lang);
+bool xccdf_policy_model_add_cpe_lang_model_source(struct xccdf_policy_model * model, struct oscap_source *source);
+
+/**
+ * Registers an additional CPE lang model for applicability testing
+ * The one embedded in the evaluated XCCDF take precedence!
+ *
+ * @deprecated Deprecated in favor of @ref xccdf_policy_model_add_cpe_lang_model_source
+ */
+OSCAP_DEPRECATED(bool xccdf_policy_model_add_cpe_lang_model(struct xccdf_policy_model * model, const char *cpe_lang));
 
 /**
  * Registers an additional CPE resource (either dictionary or language)
  * Autodetects given file and acts accordingly.
  * The one embedded in the evaluated XCCDF take precedence!
  */
-bool xccdf_policy_model_add_cpe_autodetect(struct xccdf_policy_model *model, const char* filepath);
+bool xccdf_policy_model_add_cpe_autodetect_source(struct xccdf_policy_model *model, struct oscap_source *source);
+
+/**
+ * Registers an additional CPE resource (either dictionary or language)
+ * Autodetects given file and acts accordingly.
+ * The one embedded in the evaluated XCCDF take precedence!
+ *
+ * @deprecated Deprecated in favor of @ref xccdf_policy_model_add_cpe_autodetect_source
+ */
+OSCAP_DEPRECATED(bool xccdf_policy_model_add_cpe_autodetect(struct xccdf_policy_model *model, const char *filepath));
+
+/**
+ * Retrieves an iterator of all OVAL sessions created for CPE applicability evaluation
+ * key is the OVAL href, value is the OVAL session itself (type oval_agent_session*)
+ */
+struct oscap_htable_iterator *xccdf_policy_model_get_cpe_oval_sessions(struct xccdf_policy_model *model);
 
 /**
  * Function to register callback for checking system
@@ -637,6 +691,16 @@ void xccdf_value_binding_iterator_reset(struct xccdf_value_binding_iterator *it)
  */
 struct xccdf_score * xccdf_policy_get_score(struct xccdf_policy * policy, struct xccdf_result * test_result, const char * system);
 
+/**
+ * Get value of given value item in context of given policy
+ * @memberof xccdf_policy
+ * @param policy XCCDF policy
+ * @param item the xccdf:Value to resolve
+ * @returns string representation of resolved value_instance.
+ * @retval NULL indicates failure
+ */
+const char *xccdf_policy_get_value_of_item(struct xccdf_policy * policy, struct xccdf_item * item);
+
 /**
  * Perform textual substitution of cdf:sub elements with respect to given XCCDF policy.
  * @param text text to be substituted
