@@ -1,8 +1,9 @@
 #!/bin/bash
 
-. $srcdir/../../../test_common.sh
+. ../../../test_common.sh
 
 test_init test_api_oval_unittests.log
+test_run "comment before root element" $srcdir/test_comment.sh
 test_run "empty filename(pattern match)" $srcdir/test_empty_filename.sh
 test_run "deprecated definition" $srcdir/test_deprecated_def.sh
 test_run "applicability_check element" $srcdir/test_applicability_check.sh
@@ -23,4 +24,5 @@ test_run "ipv4_address: 'subset of' operation" $srcdir/test_ipv4_subset_of.sh
 test_run "ipv4_address: comparison" $srcdir/test_ipv4_comparison.sh
 test_run "textfilecontent: 'line' comparison" $srcdir/test_filecontent_line.sh
 test_run "anyxml element" $srcdir/test_anyxml.sh
+test_run "glob to regex" $srcdir/test_glob_to_regex.sh
 test_exit
