@@ -11,8 +11,21 @@
 # Normalized path.
 PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/sbin:/usr/local/bin
 
-[ -z "$builddir" ] || export OSCAP=$(cd $builddir/utils/.libs; pwd)/oscap
-export XMLDIFF=$(cd $(dirname $BASH_SOURCE); pwd)/xmldiff.pl
+# Some of the tests rely on the "C" locale and would fail with some locales.
+LC_ALL=C
+export LC_ALL
+
+enable_valgrind=no
+if [ $enable_valgrind = "yes" ] ; then
+  actualdir=/home/slukasik-f21/data/redhat/git/hub/openscap/openscap
+  export actualdir
+  [ -z "$builddir" ] || export OSCAP="/home/slukasik-f21/data/redhat/git/hub/openscap/openscap/tests/valgrind_test.sh"
+else
+  [ -z "$builddir" ] || export OSCAP=$(cd $builddir/utils/.libs; pwd)/oscap
+fi
+
+export XMLDIFF="/home/slukasik-f21/data/redhat/git/hub/openscap/openscap/tests/xmldiff.pl"
+
 if ! XPATH=`command -v xpath 2>&1`; then
   echo "I require xpath tool but it's not installed. Aborting." >&2
   exit 1
@@ -112,8 +125,9 @@ function verify_results {
     ID=1
     while [ $ID -le $COUNT ]; do
 	
-	CON_ITEM=`grep "id=\"oval:1:${TYPE}:${ID}\"" $CONTENT`
-	RES_ITEM=`grep "${FULLTYPE}_id=\"oval:1:${TYPE}:${ID}\"" $RESULTS`	
+	CON_ITEM=`grep "id=\"oval:[[:digit:]]\+:${TYPE}:${ID}\"" $CONTENT`
+	RES_ITEM=`grep "${FULLTYPE}_id=\"oval:[[:digit:]]\+:${TYPE}:${ID}\"" $RESULTS`
+	OVAL_ID=`echo ${CON_ITEM} | grep -o "oval:[[:digit:]]\+:${TYPE}:${ID}"`
 	if (echo $RES_ITEM | grep "result=\"true\"") >/dev/null; then
 	    RES="TRUE"
 	elif (echo $RES_ITEM | grep "result=\"false\"" >/dev/null); then
@@ -131,7 +145,7 @@ function verify_results {
 	fi
 	
 	if [ ! $RES = $CMT ]; then
-	    echo "Result of oval:1:${TYPE}:${ID} should be ${CMT} and is ${RES}" 
+	    echo "Result of ${OVAL_ID} should be ${CMT} and is ${RES}"
 	    ret_val=$[$ret_val + 1]
 	fi
 	
@@ -142,7 +156,7 @@ function verify_results {
 }
 
 assert_exists() {
-        real_cnt="$($XPATH $result 'count('"$2"')')"
+        real_cnt="$($XPATH $result 'count('"$2"')' 2>/dev/null)"
         if [ "$real_cnt" != "$1" ]; then
                 echo "Failed: expected count: $1, real count: $real_cnt, xpath: '$2'"
                 return 1
