@@ -139,6 +139,7 @@ struct oscap_action {
         int doctype;
 	int force;
 	int validate;
+	int schematron;
 	int remote_resources;
 	int progress;
 	int oval_results;
@@ -151,7 +152,6 @@ struct oscap_action {
 };
 
 int app_xslt(const char *infile, const char *xsltfile, const char *outfile, const char **params);
-void validation_failed(const char *xmlfile, oscap_document_type_t doctype, const char *version);
 int reporter(const char *file, int line, const char *msg, void *arg);
 
 int oscap_module_process(struct oscap_module *module, int argc, char **argv);
@@ -160,9 +160,6 @@ int oscap_module_call(struct oscap_action *action);
 
 void oscap_print_error(void);
 
-char *oscap_acquire_temp_dir_bundled(void);
-void oscap_acquire_cleanup_dir_bundled(char **dir_path);
-
 extern struct oscap_module OSCAP_ROOT_MODULE;
 extern struct oscap_module OSCAP_DS_MODULE;
 extern struct oscap_module OSCAP_XCCDF_MODULE;
